@@ -46,6 +46,12 @@ var (
 			},
 		},
 	}
+	longerIDValidateOption = CSINodeValidationOptions{
+		AllowLongNodeID: true,
+	}
+	shorterIDValidationOption = CSINodeValidationOptions{
+		AllowLongNodeID: false,
+	}
 )
 
 func TestValidateStorageClass(t *testing.T) {
@@ -1026,8 +1032,9 @@ func TestValidateAllowedTopologies(t *testing.T) {
 func TestCSINodeValidation(t *testing.T) {
 	driverName := "driver-name"
 	driverName2 := "1io.kubernetes-storage-2-csi-driver3"
-	longName := "my-a-b-c-d-c-f-g-h-i-j-k-l-m-n-o-p-q-r-s-t-u-v-w-x-y-z-ABCDEFGHIJKLMNOPQRSTUVWXYZ-driver"
+	longName := "my-a-b-c-d-c-f-g-h-i-j-k-l-m-n-o-p-q-r-s-t-u-v-w-x-y-z-ABCDEFGHIJKLMNOPQRSTUVWXYZ-driver" // 88 chars
 	nodeID := "nodeA"
+	longID := longName + longName + "abcdefghijklmnopqrstuvwxyz" // 202 chars
 	successCases := []storage.CSINode{
 		{
 			// driver name: dot only
@@ -1217,10 +1224,29 @@ func TestCSINodeValidation(t *testing.T) {
 	}
 
 	for _, csiNode := range successCases {
-		if errs := ValidateCSINode(&csiNode); len(errs) != 0 {
+		if errs := ValidateCSINode(&csiNode, shorterIDValidationOption); len(errs) != 0 {
 			t.Errorf("expected success: %v", errs)
 		}
 	}
+
+	nodeIDCase := storage.CSINode{
+		// node ID length > 128 but < 192
+		ObjectMeta: metav1.ObjectMeta{Name: "foo7"},
+		Spec: storage.CSINodeSpec{
+			Drivers: []storage.CSINodeDriver{
+				{
+					Name:         driverName,
+					NodeID:       longID,
+					TopologyKeys: []string{"company.com/zone1", "company.com/zone2"},
+				},
+			},
+		},
+	}
+
+	if errs := ValidateCSINode(&nodeIDCase, longerIDValidateOption); len(errs) != 0 {
+		t.Errorf("expected success: %v", errs)
+	}
+
 	errorCases := []storage.CSINode{
 		{
 			// Empty driver name
@@ -1414,10 +1440,11 @@ func TestCSINodeValidation(t *testing.T) {
 				},
 			},
 		},
+		nodeIDCase,
 	}
 
 	for _, csiNode := range errorCases {
-		if errs := ValidateCSINode(&csiNode); len(errs) == 0 {
+		if errs := ValidateCSINode(&csiNode, shorterIDValidationOption); len(errs) == 0 {
 			t.Errorf("Expected failure for test: %v", csiNode)
 		}
 	}
@@ -1528,7 +1555,7 @@ func TestCSINodeUpdateValidation(t *testing.T) {
 	}
 
 	for _, csiNode := range successCases {
-		if errs := ValidateCSINodeUpdate(&csiNode, &old); len(errs) != 0 {
+		if errs := ValidateCSINodeUpdate(&csiNode, &old, shorterIDValidationOption); len(errs) != 0 {
 			t.Errorf("expected success: %+v", errs)
 		}
 	}
@@ -1651,7 +1678,7 @@ func TestCSINodeUpdateValidation(t *testing.T) {
 	}
 
 	for _, csiNode := range errorCases {
-		if errs := ValidateCSINodeUpdate(&csiNode, &old); len(errs) == 0 {
+		if errs := ValidateCSINodeUpdate(&csiNode, &old, shorterIDValidationOption); len(errs) == 0 {
 			t.Errorf("Expected failure for test: %+v", csiNode)
 		}
 	}
@@ -1665,72 +1692,95 @@ func TestCSIDriverValidation(t *testing.T) {
 	attachNotRequired := false
 	podInfoOnMount := true
 	notPodInfoOnMount := false
+	notRequiresRepublish := false
+	storageCapacity := true
+	notStorageCapacity := false
+	supportedFSGroupPolicy := storage.FileFSGroupPolicy
+	invalidFSGroupPolicy := storage.FSGroupPolicy("invalid-mode")
 	successCases := []storage.CSIDriver{
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &podInfoOnMount,
+				AttachRequired:    &attachRequired,
+				PodInfoOnMount:    &podInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 			},
 		},
 		{
 			// driver name: dot only
 			ObjectMeta: metav1.ObjectMeta{Name: "io.kubernetes.storage.csi.driver"},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &podInfoOnMount,
+				AttachRequired:    &attachRequired,
+				PodInfoOnMount:    &podInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &notStorageCapacity,
 			},
 		},
 		{
 			// driver name: dash only
 			ObjectMeta: metav1.ObjectMeta{Name: "io-kubernetes-storage-csi-driver"},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:    &attachRequired,
+				PodInfoOnMount:    &notPodInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 			},
 		},
 		{
 			// driver name: numbers
 			ObjectMeta: metav1.ObjectMeta{Name: "1csi2driver3"},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &podInfoOnMount,
+				AttachRequired:    &attachRequired,
+				PodInfoOnMount:    &podInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 			},
 		},
 		{
 			// driver name: dot and dash
 			ObjectMeta: metav1.ObjectMeta{Name: "io.kubernetes.storage.csi-driver"},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &podInfoOnMount,
+				AttachRequired:    &attachRequired,
+				PodInfoOnMount:    &podInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 			},
 		},
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:    &attachRequired,
+				PodInfoOnMount:    &notPodInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 			},
 		},
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &podInfoOnMount,
+				AttachRequired:    &attachRequired,
+				PodInfoOnMount:    &podInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 			},
 		},
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:    &attachNotRequired,
+				PodInfoOnMount:    &notPodInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 			},
 		},
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:    &attachNotRequired,
+				PodInfoOnMount:    &notPodInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 				VolumeLifecycleModes: []storage.VolumeLifecycleMode{
 					storage.VolumeLifecyclePersistent,
 				},
@@ -1739,8 +1789,10 @@ func TestCSIDriverValidation(t *testing.T) {
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:    &attachNotRequired,
+				PodInfoOnMount:    &notPodInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 				VolumeLifecycleModes: []storage.VolumeLifecycleMode{
 					storage.VolumeLifecycleEphemeral,
 				},
@@ -1749,8 +1801,10 @@ func TestCSIDriverValidation(t *testing.T) {
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:    &attachNotRequired,
+				PodInfoOnMount:    &notPodInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 				VolumeLifecycleModes: []storage.VolumeLifecycleMode{
 					storage.VolumeLifecycleEphemeral,
 					storage.VolumeLifecyclePersistent,
@@ -1760,8 +1814,10 @@ func TestCSIDriverValidation(t *testing.T) {
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:    &attachNotRequired,
+				PodInfoOnMount:    &notPodInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
 				VolumeLifecycleModes: []storage.VolumeLifecycleMode{
 					storage.VolumeLifecycleEphemeral,
 					storage.VolumeLifecyclePersistent,
@@ -1769,6 +1825,16 @@ func TestCSIDriverValidation(t *testing.T) {
 				},
 			},
 		},
+		{
+			ObjectMeta: metav1.ObjectMeta{Name: driverName},
+			Spec: storage.CSIDriverSpec{
+				AttachRequired:    &attachNotRequired,
+				PodInfoOnMount:    &notPodInfoOnMount,
+				RequiresRepublish: &notRequiresRepublish,
+				StorageCapacity:   &storageCapacity,
+				FSGroupPolicy:     &supportedFSGroupPolicy,
+			},
+		},
 	}
 
 	for _, csiDriver := range successCases {
@@ -1780,44 +1846,68 @@ func TestCSIDriverValidation(t *testing.T) {
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: invalidName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &podInfoOnMount,
+				AttachRequired:  &attachRequired,
+				PodInfoOnMount:  &podInfoOnMount,
+				StorageCapacity: &storageCapacity,
 			},
 		},
 		{
 			ObjectMeta: metav1.ObjectMeta{Name: longName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:  &attachNotRequired,
+				PodInfoOnMount:  &notPodInfoOnMount,
+				StorageCapacity: &storageCapacity,
 			},
 		},
 		{
 			// AttachRequired not set
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: nil,
-				PodInfoOnMount: &podInfoOnMount,
+				AttachRequired:  nil,
+				PodInfoOnMount:  &podInfoOnMount,
+				StorageCapacity: &storageCapacity,
 			},
 		},
 		{
-			// AttachRequired not set
+			// PodInfoOnMount not set
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: nil,
+				AttachRequired:  &attachNotRequired,
+				PodInfoOnMount:  nil,
+				StorageCapacity: &storageCapacity,
+			},
+		},
+		{
+			// StorageCapacity not set
+			ObjectMeta: metav1.ObjectMeta{Name: driverName},
+			Spec: storage.CSIDriverSpec{
+				AttachRequired:  &attachNotRequired,
+				PodInfoOnMount:  &podInfoOnMount,
+				StorageCapacity: nil,
 			},
 		},
 		{
 			// invalid mode
 			ObjectMeta: metav1.ObjectMeta{Name: driverName},
 			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+				AttachRequired:  &attachNotRequired,
+				PodInfoOnMount:  &notPodInfoOnMount,
+				StorageCapacity: &storageCapacity,
 				VolumeLifecycleModes: []storage.VolumeLifecycleMode{
 					"no-such-mode",
 				},
 			},
 		},
+		{
+			// invalid fsGroupPolicy
+			ObjectMeta: metav1.ObjectMeta{Name: driverName},
+			Spec: storage.CSIDriverSpec{
+				AttachRequired:  &attachNotRequired,
+				PodInfoOnMount:  &notPodInfoOnMount,
+				FSGroupPolicy:   &invalidFSGroupPolicy,
+				StorageCapacity: &storageCapacity,
+			},
+		},
 	}
 
 	for _, csiDriver := range errorCases {
@@ -1834,86 +1924,382 @@ func TestCSIDriverValidationUpdate(t *testing.T) {
 	attachRequired := true
 	attachNotRequired := false
 	podInfoOnMount := true
+	storageCapacity := true
 	notPodInfoOnMount := false
+	gcp := "gcp"
+	requiresRepublish := true
+	notRequiresRepublish := false
+	notStorageCapacity := false
+	resourceVersion := "1"
 	old := storage.CSIDriver{
-		ObjectMeta: metav1.ObjectMeta{Name: driverName},
+		ObjectMeta: metav1.ObjectMeta{Name: driverName, ResourceVersion: resourceVersion},
 		Spec: storage.CSIDriverSpec{
-			AttachRequired: &attachNotRequired,
-			PodInfoOnMount: &notPodInfoOnMount,
+			AttachRequired:    &attachNotRequired,
+			PodInfoOnMount:    &notPodInfoOnMount,
+			RequiresRepublish: &notRequiresRepublish,
 			VolumeLifecycleModes: []storage.VolumeLifecycleMode{
 				storage.VolumeLifecycleEphemeral,
 				storage.VolumeLifecyclePersistent,
 			},
+			StorageCapacity: &storageCapacity,
 		},
 	}
 
-	// Currently there is only one success case: exactly the same
-	// as the existing object.
-	successCases := []storage.CSIDriver{old}
-	for _, csiDriver := range successCases {
-		if errs := ValidateCSIDriverUpdate(&csiDriver, &old); len(errs) != 0 {
-			t.Errorf("expected success for %+v: %v", csiDriver, errs)
+	successCases := []struct {
+		name   string
+		modify func(new *storage.CSIDriver)
+	}{
+		{
+			name:   "no change",
+			modify: func(new *storage.CSIDriver) {},
+		},
+		{
+			name: "change TokenRequests",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.TokenRequests = []storage.TokenRequest{{Audience: gcp}}
+			},
+		},
+		{
+			name: "change RequiresRepublish",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.RequiresRepublish = &requiresRepublish
+			},
+		},
+		{
+			name: "StorageCapacity changed",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.StorageCapacity = &notStorageCapacity
+			},
+		},
+	}
+	for _, test := range successCases {
+		t.Run(test.name, func(t *testing.T) {
+			new := old.DeepCopy()
+			test.modify(new)
+			if errs := ValidateCSIDriverUpdate(new, &old); len(errs) != 0 {
+				t.Errorf("Expected success for %+v: %v", new, errs)
+			}
+		})
+	}
+
+	// Each test case changes exactly one field. None of that is valid.
+	errorCases := []struct {
+		name   string
+		modify func(new *storage.CSIDriver)
+	}{
+		{
+			name: "invalid name",
+			modify: func(new *storage.CSIDriver) {
+				new.Name = invalidName
+			},
+		},
+		{
+			name: "long name",
+			modify: func(new *storage.CSIDriver) {
+				new.Name = longName
+			},
+		},
+		{
+			name: "AttachRequired not set",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.AttachRequired = nil
+			},
+		},
+		{
+			name: "AttachRequired changed",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.AttachRequired = &attachRequired
+			},
+		},
+		{
+			name: "PodInfoOnMount not set",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.PodInfoOnMount = nil
+			},
+		},
+		{
+			name: "PodInfoOnMount changed",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.PodInfoOnMount = &podInfoOnMount
+			},
+		},
+		{
+			name: "invalid volume lifecycle mode",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.VolumeLifecycleModes = []storage.VolumeLifecycleMode{
+					"no-such-mode",
+				}
+			},
+		},
+		{
+			name: "volume lifecycle modes not set",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.VolumeLifecycleModes = nil
+			},
+		},
+		{
+			name: "VolumeLifecyclePersistent removed",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.VolumeLifecycleModes = []storage.VolumeLifecycleMode{
+					storage.VolumeLifecycleEphemeral,
+				}
+			},
+		},
+		{
+			name: "VolumeLifecycleEphemeral removed",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.VolumeLifecycleModes = []storage.VolumeLifecycleMode{
+					storage.VolumeLifecyclePersistent,
+				}
+			},
+		},
+		{
+			name: "FSGroupPolicy invalidated",
+			modify: func(new *storage.CSIDriver) {
+				invalidFSGroupPolicy := storage.FSGroupPolicy("invalid")
+				new.Spec.FSGroupPolicy = &invalidFSGroupPolicy
+			},
+		},
+		{
+			name: "FSGroupPolicy changed",
+			modify: func(new *storage.CSIDriver) {
+				fileFSGroupPolicy := storage.FileFSGroupPolicy
+				new.Spec.FSGroupPolicy = &fileFSGroupPolicy
+			},
+		},
+		{
+			name: "TokenRequests invalidated",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.TokenRequests = []storage.TokenRequest{{Audience: gcp}, {Audience: gcp}}
+			},
+		},
+		{
+			name: "invalid nil StorageCapacity",
+			modify: func(new *storage.CSIDriver) {
+				new.Spec.StorageCapacity = nil
+			},
+		},
+	}
+
+	for _, test := range errorCases {
+		t.Run(test.name, func(t *testing.T) {
+			new := old.DeepCopy()
+			test.modify(new)
+			if errs := ValidateCSIDriverUpdate(new, &old); len(errs) == 0 {
+				t.Errorf("Expected failure for test: %+v", new)
+			}
+		})
+	}
+}
+
+func TestCSIDriverStorageCapacityEnablement(t *testing.T) {
+	run := func(t *testing.T, withField bool) {
+		driverName := "test-driver"
+		attachRequired := true
+		podInfoOnMount := true
+		requiresRepublish := true
+		storageCapacity := true
+		csiDriver := storage.CSIDriver{
+			ObjectMeta: metav1.ObjectMeta{Name: driverName},
+			Spec: storage.CSIDriverSpec{
+				AttachRequired:    &attachRequired,
+				PodInfoOnMount:    &podInfoOnMount,
+				RequiresRepublish: &requiresRepublish,
+			},
+		}
+		if withField {
+			csiDriver.Spec.StorageCapacity = &storageCapacity
+		}
+		errs := ValidateCSIDriver(&csiDriver)
+		success := withField
+		if success && len(errs) != 0 {
+			t.Errorf("expected success, got: %v", errs)
+		}
+		if !success && len(errs) == 0 {
+			t.Errorf("expected error, got success")
 		}
 	}
 
-	errorCases := []storage.CSIDriver{
+	yesNo := []bool{true, false}
+	for _, withField := range yesNo {
+		t.Run(fmt.Sprintf("with-field=%v", withField), func(t *testing.T) {
+			run(t, withField)
+		})
+	}
+}
+
+func TestValidateCSIStorageCapacity(t *testing.T) {
+	storageClassName := "test-sc"
+	invalidName := "-invalid-@#$%^&*()-"
+
+	goodCapacity := storage.CSIStorageCapacity{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      "csc-329803da-fdd2-42e4-af6f-7b07e7ccc305",
+			Namespace: metav1.NamespaceDefault,
+		},
+		StorageClassName: storageClassName,
+	}
+	goodTopology := metav1.LabelSelector{
+		MatchLabels: map[string]string{"foo": "bar"},
+	}
+
+	scenarios := map[string]struct {
+		isExpectedFailure bool
+		capacity          *storage.CSIStorageCapacity
+	}{
+		"good-capacity": {
+			capacity: &goodCapacity,
+		},
+		"missing-storage-class-name": {
+			isExpectedFailure: true,
+			capacity: func() *storage.CSIStorageCapacity {
+				capacity := goodCapacity
+				capacity.StorageClassName = ""
+				return &capacity
+			}(),
+		},
+		"bad-storage-class-name": {
+			isExpectedFailure: true,
+			capacity: func() *storage.CSIStorageCapacity {
+				capacity := goodCapacity
+				capacity.StorageClassName = invalidName
+				return &capacity
+			}(),
+		},
+		"good-capacity-value": {
+			capacity: func() *storage.CSIStorageCapacity {
+				capacity := goodCapacity
+				capacity.Capacity = resource.NewQuantity(1, resource.BinarySI)
+				return &capacity
+			}(),
+		},
+		"bad-capacity-value": {
+			isExpectedFailure: true,
+			capacity: func() *storage.CSIStorageCapacity {
+				capacity := goodCapacity
+				capacity.Capacity = resource.NewQuantity(-11, resource.BinarySI)
+				return &capacity
+			}(),
+		},
+		"good-topology": {
+			capacity: func() *storage.CSIStorageCapacity {
+				capacity := goodCapacity
+				capacity.NodeTopology = &goodTopology
+				return &capacity
+			}(),
+		},
+		"empty-topology": {
+			capacity: func() *storage.CSIStorageCapacity {
+				capacity := goodCapacity
+				capacity.NodeTopology = &metav1.LabelSelector{}
+				return &capacity
+			}(),
+		},
+		"bad-topology-fields": {
+			isExpectedFailure: true,
+			capacity: func() *storage.CSIStorageCapacity {
+				capacity := goodCapacity
+				capacity.NodeTopology = &metav1.LabelSelector{
+					MatchExpressions: []metav1.LabelSelectorRequirement{
+						{
+							Key:      "foo",
+							Operator: metav1.LabelSelectorOperator("no-such-operator"),
+							Values: []string{
+								"bar",
+							},
+						},
+					},
+				}
+				return &capacity
+			}(),
+		},
+	}
+
+	for name, scenario := range scenarios {
+		t.Run(name, func(t *testing.T) {
+			errs := ValidateCSIStorageCapacity(scenario.capacity)
+			if len(errs) == 0 && scenario.isExpectedFailure {
+				t.Errorf("Unexpected success")
+			}
+			if len(errs) > 0 && !scenario.isExpectedFailure {
+				t.Errorf("Unexpected failure: %+v", errs)
+			}
+		})
+	}
+
+}
+
+func TestCSIServiceAccountToken(t *testing.T) {
+	driverName := "test-driver"
+	gcp := "gcp"
+	aws := "aws"
+	notRequiresRepublish := false
+	tests := []struct {
+		desc      string
+		csiDriver *storage.CSIDriver
+		wantErr   bool
+	}{
 		{
-			ObjectMeta: metav1.ObjectMeta{Name: invalidName},
-			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachRequired,
-				PodInfoOnMount: &podInfoOnMount,
+			desc: "invalid - TokenRequests has tokens with the same audience",
+			csiDriver: &storage.CSIDriver{
+				ObjectMeta: metav1.ObjectMeta{Name: driverName},
+				Spec: storage.CSIDriverSpec{
+					TokenRequests:     []storage.TokenRequest{{Audience: gcp}, {Audience: gcp}},
+					RequiresRepublish: &notRequiresRepublish,
+				},
 			},
+			wantErr: true,
 		},
 		{
-			ObjectMeta: metav1.ObjectMeta{Name: longName},
-			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
+			desc: "invalid - TokenRequests has tokens with ExpirationSeconds less than 10min",
+			csiDriver: &storage.CSIDriver{
+				ObjectMeta: metav1.ObjectMeta{Name: driverName},
+				Spec: storage.CSIDriverSpec{
+					TokenRequests:     []storage.TokenRequest{{Audience: gcp, ExpirationSeconds: utilpointer.Int64Ptr(10)}},
+					RequiresRepublish: &notRequiresRepublish,
+				},
 			},
+			wantErr: true,
 		},
 		{
-			// AttachRequired not set
-			ObjectMeta: metav1.ObjectMeta{Name: driverName},
-			Spec: storage.CSIDriverSpec{
-				AttachRequired: nil,
-				PodInfoOnMount: &podInfoOnMount,
+			desc: "invalid - TokenRequests has tokens with ExpirationSeconds longer than 1<<32 min",
+			csiDriver: &storage.CSIDriver{
+				ObjectMeta: metav1.ObjectMeta{Name: driverName},
+				Spec: storage.CSIDriverSpec{
+					TokenRequests:     []storage.TokenRequest{{Audience: gcp, ExpirationSeconds: utilpointer.Int64Ptr(1<<32 + 1)}},
+					RequiresRepublish: &notRequiresRepublish,
+				},
 			},
+			wantErr: true,
 		},
 		{
-			// AttachRequired not set
-			ObjectMeta: metav1.ObjectMeta{Name: driverName},
-			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: nil,
-			},
-		},
-		{
-			// invalid mode
-			ObjectMeta: metav1.ObjectMeta{Name: driverName},
-			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
-				VolumeLifecycleModes: []storage.VolumeLifecycleMode{
-					"no-such-mode",
+			desc: "valid - TokenRequests has at most one token with empty string audience",
+			csiDriver: &storage.CSIDriver{
+				ObjectMeta: metav1.ObjectMeta{Name: driverName},
+				Spec: storage.CSIDriverSpec{
+					TokenRequests:     []storage.TokenRequest{{Audience: ""}},
+					RequiresRepublish: &notRequiresRepublish,
 				},
 			},
 		},
 		{
-			// different modes
-			ObjectMeta: metav1.ObjectMeta{Name: driverName},
-			Spec: storage.CSIDriverSpec{
-				AttachRequired: &attachNotRequired,
-				PodInfoOnMount: &notPodInfoOnMount,
-				VolumeLifecycleModes: []storage.VolumeLifecycleMode{
-					storage.VolumeLifecycleEphemeral,
+			desc: "valid - TokenRequests has tokens with different audience",
+			csiDriver: &storage.CSIDriver{
+				ObjectMeta: metav1.ObjectMeta{Name: driverName},
+				Spec: storage.CSIDriverSpec{
+					TokenRequests:     []storage.TokenRequest{{}, {Audience: gcp}, {Audience: aws}},
+					RequiresRepublish: &notRequiresRepublish,
 				},
 			},
 		},
 	}
 
-	for _, csiDriver := range errorCases {
-		if errs := ValidateCSIDriverUpdate(&csiDriver, &old); len(errs) == 0 {
-			t.Errorf("Expected failure for test: %v", csiDriver)
+	for _, test := range tests {
+		test.csiDriver.Spec.AttachRequired = new(bool)
+		test.csiDriver.Spec.PodInfoOnMount = new(bool)
+		test.csiDriver.Spec.StorageCapacity = new(bool)
+		if errs := ValidateCSIDriver(test.csiDriver); test.wantErr != (len(errs) != 0) {
+			t.Errorf("ValidateCSIDriver = %v, want err: %v", errs, test.wantErr)
 		}
 	}
 }
