@@ -24,9 +24,13 @@ import (
 	discoveryv1 "k8s.io/api/discovery/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
+	genericapiserver "k8s.io/apiserver/pkg/server"
 	"k8s.io/client-go/kubernetes/fake"
+	corev1client "k8s.io/client-go/kubernetes/typed/core/v1"
+	"k8s.io/client-go/rest"
 	core "k8s.io/client-go/testing"
 	"k8s.io/kubernetes/pkg/controlplane/reconcilers"
+	corerest "k8s.io/kubernetes/pkg/registry/core/rest"
 	netutils "k8s.io/utils/net"
 )
 
@@ -994,3 +998,157 @@ func TestCreateOrUpdateMasterService(t *testing.T) {
 		}
 	}
 }
+
+func Test_completedConfig_NewBootstrapController(t *testing.T) {
+
+	_, ipv4cidr, err := netutils.ParseCIDRSloppy("192.168.0.0/24")
+	if err != nil {
+		t.Fatalf("Unexpected error %v", err)
+	}
+
+	_, ipv6cidr, err := netutils.ParseCIDRSloppy("2001:db8::/112")
+	if err != nil {
+		t.Fatalf("Unexpected error %v", err)
+	}
+
+	ipv4address := netutils.ParseIPSloppy("192.168.1.1")
+	ipv6address := netutils.ParseIPSloppy("2001:db8::1")
+
+	type args struct {
+		legacyRESTStorage corerest.LegacyRESTStorage
+		serviceClient     corev1client.ServicesGetter
+		nsClient          corev1client.NamespacesGetter
+		eventClient       corev1client.EventsGetter
+		readyzClient      rest.Interface
+	}
+	tests := []struct {
+		name        string
+		config      genericapiserver.Config
+		extraConfig *ExtraConfig
+		args        args
+		wantErr     bool
+	}{
+		{
+			name: "master endpoint reconciler - IPv4 families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.MasterCountReconcilerType,
+				ServiceIPRange:         *ipv4cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv4address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: false,
+		},
+		{
+			name: "master endpoint reconciler - IPv6 families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.MasterCountReconcilerType,
+				ServiceIPRange:         *ipv6cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv6address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: false,
+		},
+		{
+			name: "master endpoint reconciler - wrong IP families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.MasterCountReconcilerType,
+				ServiceIPRange:         *ipv4cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv6address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: true,
+		},
+		{
+			name: "master endpoint reconciler - wrong IP families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.MasterCountReconcilerType,
+				ServiceIPRange:         *ipv6cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv4address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: true,
+		},
+		{
+			name: "lease endpoint reconciler - IPv4 families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.LeaseEndpointReconcilerType,
+				ServiceIPRange:         *ipv4cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv4address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: false,
+		},
+		{
+			name: "lease endpoint reconciler - IPv6 families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.LeaseEndpointReconcilerType,
+				ServiceIPRange:         *ipv6cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv6address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: false,
+		},
+		{
+			name: "lease endpoint reconciler - wrong IP families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.LeaseEndpointReconcilerType,
+				ServiceIPRange:         *ipv4cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv6address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: true,
+		},
+		{
+			name: "lease endpoint reconciler - wrong IP families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.LeaseEndpointReconcilerType,
+				ServiceIPRange:         *ipv6cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv4address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: true,
+		},
+		{
+			name: "none endpoint reconciler - wrong IP families",
+			extraConfig: &ExtraConfig{
+				EndpointReconcilerType: reconcilers.NoneEndpointReconcilerType,
+				ServiceIPRange:         *ipv4cidr,
+			},
+			config: genericapiserver.Config{
+				PublicAddress: ipv6address,
+				SecureServing: &genericapiserver.SecureServingInfo{Listener: fakeLocalhost443Listener{}},
+			},
+			wantErr: false,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			c := &completedConfig{
+				GenericConfig: tt.config.Complete(nil),
+				ExtraConfig:   tt.extraConfig,
+			}
+			_, err := c.NewBootstrapController(tt.args.legacyRESTStorage, tt.args.serviceClient, tt.args.nsClient, tt.args.eventClient, tt.args.readyzClient)
+			if (err != nil) != tt.wantErr {
+				t.Errorf("completedConfig.NewBootstrapController() error = %v, wantErr %v", err, tt.wantErr)
+				return
+			}
+
+		})
+	}
+}
