@@ -34,10 +34,8 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	clientset "k8s.io/client-go/kubernetes"
 	"k8s.io/component-helpers/storage/ephemeral"
-	"k8s.io/kubernetes/pkg/features"
 	"k8s.io/kubernetes/pkg/kubelet/config"
 	kubecontainer "k8s.io/kubernetes/pkg/kubelet/container"
 	"k8s.io/kubernetes/pkg/kubelet/pod"
@@ -192,24 +190,29 @@ func (dswp *desiredStateOfWorldPopulator) populatorLoop() {
 func (dswp *desiredStateOfWorldPopulator) findAndAddNewPods() {
 	// Map unique pod name to outer volume name to MountedVolume.
 	mountedVolumesForPod := make(map[volumetypes.UniquePodName]map[string]cache.MountedVolume)
-	if utilfeature.DefaultFeatureGate.Enabled(features.ExpandInUsePersistentVolumes) {
-		for _, mountedVolume := range dswp.actualStateOfWorld.GetMountedVolumes() {
-			mountedVolumes, exist := mountedVolumesForPod[mountedVolume.PodName]
-			if !exist {
-				mountedVolumes = make(map[string]cache.MountedVolume)
-				mountedVolumesForPod[mountedVolume.PodName] = mountedVolumes
-			}
-			mountedVolumes[mountedVolume.OuterVolumeSpecName] = mountedVolume
+	for _, mountedVolume := range dswp.actualStateOfWorld.GetMountedVolumes() {
+		mountedVolumes, exist := mountedVolumesForPod[mountedVolume.PodName]
+		if !exist {
+			mountedVolumes = make(map[string]cache.MountedVolume)
+			mountedVolumesForPod[mountedVolume.PodName] = mountedVolumes
 		}
+		mountedVolumes[mountedVolume.OuterVolumeSpecName] = mountedVolume
 	}
 
-	processedVolumesForFSResize := sets.NewString()
 	for _, pod := range dswp.podManager.GetPods() {
-		if dswp.podStateProvider.ShouldPodContainersBeTerminating(pod.UID) {
+		// Keep consistency of adding pod during reconstruction
+		if dswp.hasAddedPods && dswp.podStateProvider.ShouldPodContainersBeTerminating(pod.UID) {
 			// Do not (re)add volumes for pods that can't also be starting containers
 			continue
 		}
-		dswp.processPodVolumes(pod, mountedVolumesForPod, processedVolumesForFSResize)
+
+		if !dswp.hasAddedPods && dswp.podStateProvider.ShouldPodRuntimeBeRemoved(pod.UID) {
+			// When kubelet restarts, we need to add pods to dsw if there is a possibility
+			// that the container may still be running
+			continue
+		}
+
+		dswp.processPodVolumes(pod, mountedVolumesForPod)
 	}
 }
 
@@ -277,8 +280,7 @@ func (dswp *desiredStateOfWorldPopulator) findAndRemoveDeletedPods() {
 // desired state of the world.
 func (dswp *desiredStateOfWorldPopulator) processPodVolumes(
 	pod *v1.Pod,
-	mountedVolumesForPod map[volumetypes.UniquePodName]map[string]cache.MountedVolume,
-	processedVolumesForFSResize sets.String) {
+	mountedVolumesForPod map[volumetypes.UniquePodName]map[string]cache.MountedVolume) {
 	if pod == nil {
 		return
 	}
@@ -291,7 +293,6 @@ func (dswp *desiredStateOfWorldPopulator) processPodVolumes(
 	allVolumesAdded := true
 	mounts, devices := util.GetPodVolumeNames(pod)
 
-	expandInUsePV := utilfeature.DefaultFeatureGate.Enabled(features.ExpandInUsePersistentVolumes)
 	// Process volume spec for each volume defined in pod
 	for _, podVolume := range pod.Spec.Volumes {
 		if !mounts.Has(podVolume.Name) && !devices.Has(podVolume.Name) {
@@ -322,10 +323,7 @@ func (dswp *desiredStateOfWorldPopulator) processPodVolumes(
 		// sync reconstructed volume
 		dswp.actualStateOfWorld.SyncReconstructedVolume(uniqueVolumeName, uniquePodName, podVolume.Name)
 
-		if expandInUsePV {
-			dswp.checkVolumeFSResize(pod, podVolume, pvc, volumeSpec,
-				uniquePodName, mountedVolumesForPod, processedVolumesForFSResize)
-		}
+		dswp.checkVolumeFSResize(pod, podVolume, pvc, volumeSpec, uniquePodName, mountedVolumesForPod)
 	}
 
 	// some of the volume additions may have failed, should not mark this pod as fully processed
@@ -346,21 +344,16 @@ func (dswp *desiredStateOfWorldPopulator) processPodVolumes(
 
 }
 
-// checkVolumeFSResize checks whether a PVC mounted by the pod requires file
-// system resize or not. If so, marks this volume as fsResizeRequired in ASW.
-//   - mountedVolumesForPod stores all mounted volumes in ASW, because online
-//     volume resize only considers mounted volumes.
-//   - processedVolumesForFSResize stores all volumes we have checked in current loop,
-//     because file system resize operation is a global operation for volume, so
-//     we only need to check it once if more than one pod use it.
+// checkVolumeFSResize records desired PVC size for a volume mounted by the pod.
+// It is used for comparison with actual size(coming from pvc.Status.Capacity) and calling
+// volume expansion on the node if needed.
 func (dswp *desiredStateOfWorldPopulator) checkVolumeFSResize(
 	pod *v1.Pod,
 	podVolume v1.Volume,
 	pvc *v1.PersistentVolumeClaim,
 	volumeSpec *volume.Spec,
 	uniquePodName volumetypes.UniquePodName,
-	mountedVolumesForPod map[volumetypes.UniquePodName]map[string]cache.MountedVolume,
-	processedVolumesForFSResize sets.String) {
+	mountedVolumesForPod map[volumetypes.UniquePodName]map[string]cache.MountedVolume) {
 
 	// if a volumeSpec does not have PV or has InlineVolumeSpecForCSIMigration set or pvc is nil
 	// we can't resize the volume and hence resizing should be skipped.
@@ -376,11 +369,6 @@ func (dswp *desiredStateOfWorldPopulator) checkVolumeFSResize(
 		// or online resize in subsequent loop(after we confirm it has been mounted).
 		return
 	}
-	if processedVolumesForFSResize.Has(string(uniqueVolumeName)) {
-		// File system resize operation is a global operation for volume,
-		// so we only need to check it once if more than one pod use it.
-		return
-	}
 	// volumeSpec.ReadOnly is the value that determines if volume could be formatted when being mounted.
 	// This is the same flag that determines filesystem resizing behaviour for offline resizing and hence
 	// we should use it here. This value comes from Pod.spec.volumes.persistentVolumeClaim.readOnly.
@@ -389,10 +377,12 @@ func (dswp *desiredStateOfWorldPopulator) checkVolumeFSResize(
 		klog.V(5).InfoS("Skip file system resize check for the volume, as the volume is mounted as readonly", "pod", klog.KObj(pod), "volumeName", podVolume.Name)
 		return
 	}
-	if volumeRequiresFSResize(pvc, volumeSpec.PersistentVolume) {
-		dswp.actualStateOfWorld.MarkFSResizeRequired(uniqueVolumeName, uniquePodName)
-	}
-	processedVolumesForFSResize.Insert(string(uniqueVolumeName))
+	pvCap := volumeSpec.PersistentVolume.Spec.Capacity.Storage()
+	pvcStatusCap := pvc.Status.Capacity.Storage()
+	dswp.desiredStateOfWorld.UpdatePersistentVolumeSize(uniqueVolumeName, pvCap)
+
+	// in case the actualStateOfWorld was rebuild after kubelet restart ensure that claimSize is set to accurate value
+	dswp.actualStateOfWorld.InitializeClaimSize(uniqueVolumeName, pvcStatusCap)
 }
 
 func getUniqueVolumeName(
@@ -410,12 +400,6 @@ func getUniqueVolumeName(
 	return mountedVolume.VolumeName, true
 }
 
-func volumeRequiresFSResize(pvc *v1.PersistentVolumeClaim, pv *v1.PersistentVolume) bool {
-	capacity := pvc.Status.Capacity[v1.ResourceStorage]
-	requested := pv.Spec.Capacity[v1.ResourceStorage]
-	return requested.Cmp(capacity) > 0
-}
-
 // podPreviouslyProcessed returns true if the volumes for this pod have already
 // been processed/reprocessed by the populator. Otherwise, the volumes for this pod need to
 // be reprocessed.
