@@ -23,6 +23,7 @@ import (
 	"sync"
 	"time"
 
+	apirequest "k8s.io/apiserver/pkg/endpoints/request"
 	compbasemetrics "k8s.io/component-base/metrics"
 	"k8s.io/component-base/metrics/legacyregistry"
 	basemetricstestutil "k8s.io/component-base/metrics/testutil"
@@ -89,7 +90,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "rejected_requests_total",
-			Help:           "Number of requests rejected by API Priority and Fairness system",
+			Help:           "Number of requests rejected by API Priority and Fairness subsystem",
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
 		[]string{priorityLevel, flowSchema, "reason"},
@@ -99,7 +100,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "dispatched_requests_total",
-			Help:           "Number of requests released by API Priority and Fairness system for service",
+			Help:           "Number of requests executed by API Priority and Fairness subsystem",
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
 		[]string{priorityLevel, flowSchema},
@@ -110,7 +111,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "priority_level_seat_count_samples",
-			Help:           "Periodic observations of the number of requests",
+			Help:           "Periodic observations of number of seats occupied for any stage of execution (but only initial stage for WATCHes)",
 			Buckets:        []float64{0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1},
 			ConstLabels:    map[string]string{phase: "executing"},
 			StabilityLevel: compbasemetrics.ALPHA,
@@ -119,7 +120,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "priority_level_seat_count_watermarks",
-			Help:           "Watermarks of the number of requests",
+			Help:           "Watermarks of the number of seats occupied for any stage of execution (but only initial stage for WATCHes)",
 			Buckets:        []float64{0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1},
 			ConstLabels:    map[string]string{phase: "executing"},
 			StabilityLevel: compbasemetrics.ALPHA,
@@ -132,7 +133,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "priority_level_request_count_samples",
-			Help:           "Periodic observations of the number of requests",
+			Help:           "Periodic observations of the number of requests waiting or in any stage of execution (but only initial stage for WATCHes)",
 			Buckets:        []float64{0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1},
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
@@ -140,7 +141,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "priority_level_request_count_watermarks",
-			Help:           "Watermarks of the number of requests",
+			Help:           "Watermarks of the number of requests waiting or in any stage of execution (but only initial stage for WATCHes)",
 			Buckets:        []float64{0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1},
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
@@ -152,7 +153,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "read_vs_write_request_count_samples",
-			Help:           "Periodic observations of the number of requests",
+			Help:           "Periodic observations of the number of requests waiting or in regular stage of execution",
 			Buckets:        []float64{0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1},
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
@@ -160,7 +161,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "read_vs_write_request_count_watermarks",
-			Help:           "Watermarks of the number of requests",
+			Help:           "Watermarks of the number of requests waiting or in regular stage of execution",
 			Buckets:        []float64{0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1},
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
@@ -221,7 +222,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "current_inqueue_requests",
-			Help:           "Number of requests currently pending in queues of the API Priority and Fairness system",
+			Help:           "Number of requests currently pending in queues of the API Priority and Fairness subsystem",
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
 		[]string{priorityLevel, flowSchema},
@@ -231,7 +232,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "request_queue_length_after_enqueue",
-			Help:           "Length of queue in the API Priority and Fairness system, as seen by each request after it is enqueued",
+			Help:           "Length of queue in the API Priority and Fairness subsystem, as seen by each request after it is enqueued",
 			Buckets:        queueLengthBuckets,
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
@@ -242,7 +243,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "request_concurrency_limit",
-			Help:           "Shared concurrency limit in the API Priority and Fairness system",
+			Help:           "Shared concurrency limit in the API Priority and Fairness subsystem",
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
 		[]string{priorityLevel},
@@ -252,7 +253,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "current_executing_requests",
-			Help:           "Number of requests in regular execution phase in the API Priority and Fairness system",
+			Help:           "Number of requests in initial (for a WATCH) or any (for a non-WATCH) execution stage in the API Priority and Fairness subsystem",
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
 		[]string{priorityLevel, flowSchema},
@@ -262,7 +263,7 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "request_concurrency_in_use",
-			Help:           "Concurrency (number of seats) occupided by the currently executing (all phases count) requests in the API Priority and Fairness system",
+			Help:           "Concurrency (number of seats) occupied by the currently executing (initial stage for a WATCH, any stage otherwise) requests in the API Priority and Fairness subsystem",
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
 		[]string{priorityLevel, flowSchema},
@@ -283,11 +284,11 @@ var (
 			Namespace:      namespace,
 			Subsystem:      subsystem,
 			Name:           "request_execution_seconds",
-			Help:           "Duration of regular phase of request execution in the API Priority and Fairness system",
+			Help:           "Duration of initial stage (for a WATCH) or any (for a non-WATCH) stage of request execution in the API Priority and Fairness subsystem",
 			Buckets:        requestDurationSecondsBuckets,
 			StabilityLevel: compbasemetrics.ALPHA,
 		},
-		[]string{priorityLevel, flowSchema},
+		[]string{priorityLevel, flowSchema, "type"},
 	)
 	watchCountSamples = compbasemetrics.NewHistogramVec(
 		&compbasemetrics.HistogramOpts{
@@ -310,6 +311,29 @@ var (
 		},
 		[]string{priorityLevel, "success"},
 	)
+	apiserverWorkEstimatedSeats = compbasemetrics.NewHistogramVec(
+		&compbasemetrics.HistogramOpts{
+			Namespace: namespace,
+			Subsystem: subsystem,
+			Name:      "work_estimated_seats",
+			Help:      "Number of estimated seats (maximum of initial and final seats) associated with requests in API Priority and Fairness",
+			// the upper bound comes from the maximum number of seats a request
+			// can occupy which is currently set at 10.
+			Buckets:        []float64{1, 2, 4, 10},
+			StabilityLevel: compbasemetrics.ALPHA,
+		},
+		[]string{priorityLevel, flowSchema},
+	)
+	apiserverDispatchWithNoAccommodation = compbasemetrics.NewCounterVec(
+		&compbasemetrics.CounterOpts{
+			Namespace:      namespace,
+			Subsystem:      subsystem,
+			Name:           "request_dispatch_no_accommodation_total",
+			Help:           "Number of times a dispatch attempt resulted in a non accommodation due to lack of available seats",
+			StabilityLevel: compbasemetrics.ALPHA,
+		},
+		[]string{priorityLevel, flowSchema},
+	)
 
 	metrics = Registerables{
 		apiserverRejectedRequestsTotal,
@@ -328,6 +352,8 @@ var (
 		apiserverRequestExecutionSeconds,
 		watchCountSamples,
 		apiserverEpochAdvances,
+		apiserverWorkEstimatedSeats,
+		apiserverDispatchWithNoAccommodation,
 	}.
 		Append(PriorityLevelExecutionSeatsObserverGenerator.metrics()...).
 		Append(PriorityLevelConcurrencyObserverPairGenerator.metrics()...).
@@ -392,7 +418,11 @@ func ObserveWaitingDuration(ctx context.Context, priorityLevel, flowSchema, exec
 
 // ObserveExecutionDuration observes the execution duration for flow control
 func ObserveExecutionDuration(ctx context.Context, priorityLevel, flowSchema string, executionTime time.Duration) {
-	apiserverRequestExecutionSeconds.WithContext(ctx).WithLabelValues(priorityLevel, flowSchema).Observe(executionTime.Seconds())
+	reqType := "regular"
+	if requestInfo, ok := apirequest.RequestInfoFrom(ctx); ok && requestInfo.Verb == "watch" {
+		reqType = requestInfo.Verb
+	}
+	apiserverRequestExecutionSeconds.WithContext(ctx).WithLabelValues(priorityLevel, flowSchema, reqType).Observe(executionTime.Seconds())
 }
 
 // ObserveWatchCount notes a sampling of a watch count
@@ -404,3 +434,14 @@ func ObserveWatchCount(ctx context.Context, priorityLevel, flowSchema string, co
 func AddEpochAdvance(ctx context.Context, priorityLevel string, success bool) {
 	apiserverEpochAdvances.WithContext(ctx).WithLabelValues(priorityLevel, strconv.FormatBool(success)).Inc()
 }
+
+// ObserveWorkEstimatedSeats notes a sampling of estimated seats associated with a request
+func ObserveWorkEstimatedSeats(priorityLevel, flowSchema string, seats int) {
+	apiserverWorkEstimatedSeats.WithLabelValues(priorityLevel, flowSchema).Observe(float64(seats))
+}
+
+// AddDispatchWithNoAccommodation keeps track of number of times dispatch attempt results
+// in a non accommodation due to lack of available seats.
+func AddDispatchWithNoAccommodation(priorityLevel, flowSchema string) {
+	apiserverDispatchWithNoAccommodation.WithLabelValues(priorityLevel, flowSchema).Inc()
+}
